/*
 * Copyright (c) 2019 Asian Information Technology Group
 * All Rights Reserved.
 * NOTICE: USING THIS PROGRAM SOURCE CODE DEEMS TO ACCEPT THE FOLLOWING DISCLAIMER:
 * WE PROVIDE THIS SOFTWARE FOR YOUR REFERENCE ONLY AND EXPRESSLY DISCLAIM ALL EXPRESS, IMPLIED, STATUTORY AND OTHER WARRANTIES OR
 * REPRESENTATIONS, INCLUDING BUT NOT LIMITED TO, THE WARRANTIES OF VIRUS-FREE, PROGRAM ERROR-FREE, MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF INTELLECTUAL PROPERTY RIGHTS. WHETHER OR NOT YOU USE THIS SOFTWARE PROGRAM IS AT YOUR
 * DISCRETION AND  WE SHALL BEIN NO EVENT LIABLE FOR ANY DIRECT OR INDIRECT DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE (INCLUDING BUT
 * NOT LIMITED TO, LOSS OF OPERATING INCOME, PROFITS OR ECONOMIC BENEFITS, LOSS OF DATA OR BUSINESS INTERRUPTION, ETC.), REGARDLESS WHETHER
 * OR NOT  THE DAMAGE IS PREDICTABLE.
 */
/*
 * AITg_button.c
 *
 *  Created on: 2019~1016
 *      Author: AITg
 *
 *	\໡
 *  ϥζ}oWS6sӷ@LED}ϥΡAS6Uɶ}oWTRI-COLOR LEDO|GAS6}ɶ}oWTRI-COLOR LEDO|
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "qurt_types.h"
#include "qcli.h"
#include "qcli_api.h"
#include "qapi_tlmm.h"
#include "qapi_timer.h"
#include "qurt_thread.h"
#include "qurt_signal.h"
#include "qapi_gpioint.h"
#include "qapi_delay.h"
#include "AITg_humidity_temperature.h"

//Button PIN for Kit
#define GPIO_NUM_BUTTON_PIN       	    10
//GPIO 11}챵aAϥGPIO 10ӷ}s
#define GPIO_NUM_BUTTON_HIGH       	    11
//LED PIN for Kit
#define GPIO_NUM_GREEN_LIGHT       	    20

static qapi_Instance_Handle_t Button_Int_hdl;

//CGPIOnU۪GPIO ID
qapi_GPIO_ID_t gpio_id_green;
qapi_GPIO_ID_t gpio_id_button_int;
qapi_GPIO_ID_t gpio_id_pin_high;

qurt_signal_t Button_Interrupt_signal;
//signal mark
uint32_t Button_INTR_EVENT = (1<<0);

QCLI_Group_Handle_t qcli_button_handle;
const QCLI_Command_Group_t button_cmd_group =
{
  "AITg_button",
	0,
	NULL
};

static void ButtonIntr(qapi_GPIOINT_Callback_Data_t pin)
{
	qurt_signal_set(&Button_Interrupt_signal, Button_INTR_EVENT);
}

void Button_Thread( void *data)
{
   qapi_Status_t status = QAPI_ERROR;
   while(true)
   {
	   qurt_signal_wait(&Button_Interrupt_signal, Button_INTR_EVENT, QURT_SIGNAL_ATTR_CLEAR_MASK);
	   if(qapi_TLMM_Read_Gpio(gpio_id_button_int, GPIO_NUM_BUTTON_PIN) == QAPI_GPIO_LOW_VALUE_E)
	   {
			status = qapi_TLMM_Drive_Gpio(gpio_id_green, GPIO_NUM_GREEN_LIGHT, QAPI_GPIO_HIGH_VALUE_E);
			if (status != QAPI_OK)
			{
					QCLI_Printf(qcli_button_handle, "GPIO_NUM_RED_LIGHT qapi_TLMM_Drive_Gpio High Fail \n");
			}
		   if(Sensors_Humidity_Temperature_Get_Measured_Value() != QAPI_OK)
			   QCLI_Printf(qcli_button_handle, " Sensors_Humidity_Temperature_Get_Measured_Value NO_OK\n");
	   }
	   else
	   {
			status = qapi_TLMM_Drive_Gpio(gpio_id_green, GPIO_NUM_GREEN_LIGHT, QAPI_GPIO_LOW_VALUE_E);
			if (status != QAPI_OK)
			{
					QCLI_Printf(qcli_button_handle, "GPIO_NUM_RED_LIGHT qapi_TLMM_Drive_Gpio low Fail \n");
			}
	   }
	   qapi_Task_Delay(100*1000);	//delay 100 ms
   }
}

void Button_Thread_Init(void)
{
	qurt_thread_attr_t Thread_Attribte;
	qurt_thread_t      Thread_Handle;
	qapi_Status_t                Result;
	/* Start the main demo thread. */
	qurt_thread_attr_init(&Thread_Attribte);
	qurt_thread_attr_set_name(&Thread_Attribte, "Button_Thread");
	qurt_thread_attr_set_priority(&Thread_Attribte, 15);
	qurt_thread_attr_set_stack_size(&Thread_Attribte, 1024);
	Result = qurt_thread_create(&Thread_Handle, &Thread_Attribte, Button_Thread, NULL);
	if(Result != QAPI_OK)
	{
		QCLI_Printf(qcli_button_handle,"Failed to start Button_Thread_Init. \r\n");
	}
  if (qurt_signal_init(&Button_Interrupt_signal) != QAPI_OK)
  {
  	QCLI_Printf(qcli_button_handle,"Not able to initialize signal\n");
  }
}

qapi_Status_t Initialize_Button_Led_GPIO(void)
{
	//Create GPIO
  qapi_TLMM_Config_t tlmm_config;
  qapi_Status_t status = QAPI_ERROR;

	tlmm_config.pin = GPIO_NUM_GREEN_LIGHT;
	tlmm_config.func = 0;
	tlmm_config.dir = QAPI_GPIO_OUTPUT_E;
	tlmm_config.pull = QAPI_GPIO_NO_PULL_E;
	tlmm_config.drive = QAPI_GPIO_16MA_E;
	status = qapi_TLMM_Get_Gpio_ID( &tlmm_config, &gpio_id_green);
	if (status == QAPI_OK)
	{
		 status = qapi_TLMM_Config_Gpio(gpio_id_green, &tlmm_config);
		 if (status != QAPI_OK)
		 {
			 QCLI_Printf(qcli_button_handle, "GPIO_NUM_RED_LIGHT Fail to qapi_TLMM_Config_Gpio");
			 return QAPI_ERROR;
		 }
	}
	else
		 return QAPI_ERROR;

	return QAPI_OK;
}

qapi_Status_t Initialize_Button_Interrupt_GPIO(void)
{
	//Create GPIO Interrupt
	qapi_TLMM_Config_t tlmm_config;
	qapi_Status_t status = QAPI_ERROR;

	//GPIO 11}챵aAϥGPIO 10ӷ}s
	tlmm_config.pin = GPIO_NUM_BUTTON_HIGH;
	tlmm_config.func = 0;
	tlmm_config.dir = QAPI_GPIO_OUTPUT_E;
	tlmm_config.pull = QAPI_GPIO_NO_PULL_E;
	tlmm_config.drive = QAPI_GPIO_16MA_E;
	status = qapi_TLMM_Get_Gpio_ID( &tlmm_config, &gpio_id_pin_high);
	if (status == QAPI_OK)
	{
		 status = qapi_TLMM_Config_Gpio(gpio_id_pin_high, &tlmm_config);
		 if (status != QAPI_OK)
		 {
			 QCLI_Printf(qcli_button_handle, "GPIO_NUM_BUTTON_LOW Fail to qapi_TLMM_Config_Gpio");
			 return QAPI_ERROR;
		 }
	}
	else
	{
		QCLI_Printf(qcli_button_handle, "GPIO_NUM_BUTTON_HIGH qapi_TLMM_Get_Gpio_ID Fail");
		 return QAPI_ERROR;
	}
	status = qapi_TLMM_Drive_Gpio(gpio_id_pin_high, GPIO_NUM_BUTTON_HIGH, QAPI_GPIO_LOW_VALUE_E);
	if (status != QAPI_OK)
	{
			QCLI_Printf(qcli_button_handle, "GPIO_NUM_BUTTON_LOW qapi_TLMM_Drive_Gpio low Fail \n");
	}

	tlmm_config.pin = GPIO_NUM_BUTTON_PIN;
	tlmm_config.func = 0;   // Using the functionality tied to pin mux value
	tlmm_config.dir =  QAPI_GPIO_INPUT_E;
	tlmm_config.pull =  QAPI_GPIO_PULL_UP_E;
	tlmm_config.drive = QAPI_GPIO_2MA_E;
	status = qapi_TLMM_Get_Gpio_ID(&tlmm_config, &gpio_id_button_int);
	if (status == QAPI_OK)
	{
		status = qapi_TLMM_Config_Gpio(gpio_id_button_int, &tlmm_config);
		if (status != QAPI_OK)
		{
			QCLI_Printf(qcli_button_handle, "GPIO_NUM_BUTTON_PIN Fail to qapi_TLMM_Config_Gpio \n");
			return QAPI_ERROR;
		}
	}
	else
	{
		QCLI_Printf(qcli_button_handle, "GPIO_NUM_BUTTON_PIN qapi_TLMM_Get_Gpio_ID Fail");
		 return QAPI_ERROR;
	}

	if ( qapi_GPIOINT_Register_Interrupt(&Button_Int_hdl, GPIO_NUM_BUTTON_PIN, (qapi_GPIOINT_CB_t)ButtonIntr,
			0,  QAPI_GPIOINT_TRIGGER_EDGE_DUAL_E, QAPI_GPIOINT_PRIO_MEDIUM_E, false) != QAPI_OK)
	{
		QCLI_Printf(qcli_button_handle,"Error in GPIO_NUM_BUTTON_PIN Registering Interrupt! \n");
		return QAPI_ERROR;
	}
	QCLI_Printf(qcli_button_handle,"Initialize_Button_Interrupt_GPIO Ok! \n");
	return QAPI_OK;
}

void Initialize_Button_Demo(void)
{
	qcli_button_handle = QCLI_Register_Command_Group(NULL, &button_cmd_group);
	if (qcli_button_handle)
	{
			QCLI_Printf(qcli_button_handle, "Button Registered\n");
	}
	if(Initialize_Button_Led_GPIO() != QAPI_OK)
		QCLI_Printf(qcli_button_handle, "Initialize_Button_Led_GPIO fail \n");

	Button_Thread_Init();

	if(Initialize_Button_Interrupt_GPIO() != QAPI_OK)
		QCLI_Printf(qcli_button_handle, "Initialize_Button_Interrupt_GPIO fail \n");
}
