/*
 * Copyright (c) 2019 Asian Information Technology Group
 * All Rights Reserved.
 * NOTICE: USING THIS PROGRAM SOURCE CODE DEEMS TO ACCEPT THE FOLLOWING DISCLAIMER:
 * WE PROVIDE THIS SOFTWARE FOR YOUR REFERENCE ONLY AND EXPRESSLY DISCLAIM ALL EXPRESS, IMPLIED, STATUTORY AND OTHER WARRANTIES OR
 * REPRESENTATIONS, INCLUDING BUT NOT LIMITED TO, THE WARRANTIES OF VIRUS-FREE, PROGRAM ERROR-FREE, MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF INTELLECTUAL PROPERTY RIGHTS. WHETHER OR NOT YOU USE THIS SOFTWARE PROGRAM IS AT YOUR
 * DISCRETION AND  WE SHALL BEIN NO EVENT LIABLE FOR ANY DIRECT OR INDIRECT DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE (INCLUDING BUT
 * NOT LIMITED TO, LOSS OF OPERATING INCOME, PROFITS OR ECONOMIC BENEFITS, LOSS OF DATA OR BUSINESS INTERRUPTION, ETC.), REGARDLESS WHETHER
 * OR NOT  THE DAMAGE IS PREDICTABLE.
 */
/*
 * AITg_humidity_temperature.c
 *
 *  Created on: 2019~1016
 *      Author: AITg
 *
 *	\໡
 *  }ɷ|Slave]AddressǡAϥS6s@UiHzLI2Cӱor}oWU73(HTS221TR)Sensor humidity and temperature .
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "qurt_types.h"
#include "qcli.h"
#include "qcli_api.h"
#include "qapi_tlmm.h"
#include "qapi_timer.h"
#include "qurt_thread.h"
#include "qurt_signal.h"
#include "qurt_timer.h"
#include "AITg_humidity_temperature.h"
#include <qapi_i2c_master.h>
#include "qapi_delay.h"

QCLI_Group_Handle_t 	qcli_humidity_temperature_handle;
const QCLI_Command_Group_t humidity_temperature_cmd_group =
{
	"AITg_humidity_temperature",
	0,
	NULL
};

//I2C information
void *h1;
qurt_signal_t   i2c_ready_signal;
#define  I2CM_READY_SIG_MASK         0x10

qapi_I2CM_Config_t config_humidity_temperature =
{
    100,            										/**< I2C bus speed in kHz. */
    I2C_ADDR_HUMIDITY_TEMPERATURE,      /**< 7-bit I2C slave address. */
    0,                  								/**< SMBUS mode transfers. Set to TRUE for SMBUS mode. */
    0,   																/**< Maximum slave clock stretch in us that a slave might perform. */
    0,        													/**< Core Specific Configuration. Recommended 0. */
    0,          												/**< Core Specific Configuration. Recommended 0. */
};

void I2CM_Transfer_cb(const uint32_t status, void *CB_Parameter)
{
   qurt_signal_set(&i2c_ready_signal, I2CM_READY_SIG_MASK);
}

uint8_t write_sensor_reg8(qapi_I2CM_Config_t *pI2C_config, uint8 reg_addr_val, uint8_t byte_val)
{
	qapi_Status_t status;
	qapi_I2CM_Descriptor_t desc;
	uint8 wr_buf[2];

	wr_buf[0] = reg_addr_val;
	wr_buf[1] = byte_val;

	desc.buffer = wr_buf;
	desc.length = 2;
	desc.transferred = 0;
	desc.flags = QAPI_I2C_FLAG_START | QAPI_I2C_FLAG_STOP | QAPI_I2C_FLAG_WRITE;
	status = qapi_I2CM_Transfer(h1, pI2C_config, &desc, 1, I2CM_Transfer_cb, NULL);
	if (status != QAPI_OK)
	{
		return 0;
	}
	qurt_signal_wait(&i2c_ready_signal, I2CM_READY_SIG_MASK, QURT_SIGNAL_ATTR_CLEAR_MASK);
	return 0;
}

void write_humidity_temperature_reg(uint32_t reg_addr, uint8_t reg_val)
{
	if (reg_addr != HUMIDITY_TEMPERATURE_I2C_REG_ADDR_AV_CONF && reg_addr != HUMIDITY_TEMPERATURE_I2C_REG_ADDR_CTRL_1 && reg_addr != HUMIDITY_TEMPERATURE_I2C_REG_ADDR_CTRL_2 && reg_addr != HUMIDITY_TEMPERATURE_I2C_REG_ADDR_CTRL_3)
	{
		QCLI_Printf(qcli_humidity_temperature_handle, "Humidity Temperature wrong Register Addr:%x\n", reg_addr);
		return;
	}
	write_sensor_reg8(&config_humidity_temperature, reg_addr, reg_val);

	QCLI_Printf(qcli_humidity_temperature_handle, "Humidity Temperature Sensor Reg:0x%X  Val:0x%X\n", reg_addr, reg_val);
	return;
}

uint8_t read_sensor_reg8(qapi_I2CM_Config_t *pI2C_config, uint8 reg_addr_val)
{
	qapi_Status_t status;
	uint8 reg_addr, reg_val;
	qapi_I2CM_Descriptor_t desc;

	desc.buffer = &reg_addr;
	desc.length = 1;
	desc.transferred = 0;
	desc.flags = QAPI_I2C_FLAG_START | QAPI_I2C_FLAG_STOP | QAPI_I2C_FLAG_WRITE;
	reg_addr = reg_addr_val;
	status = qapi_I2CM_Transfer(h1, pI2C_config, &desc, 1, I2CM_Transfer_cb, NULL);

	if (status != QAPI_OK)
	{
        return 0;
	}

	qurt_signal_wait(&i2c_ready_signal, I2CM_READY_SIG_MASK, QURT_SIGNAL_ATTR_CLEAR_MASK);

	desc.buffer = &reg_val;
	desc.length = 1;
	desc.transferred = 0;
	desc.flags = QAPI_I2C_FLAG_START | QAPI_I2C_FLAG_STOP | QAPI_I2C_FLAG_READ;
	status = qapi_I2CM_Transfer(h1, pI2C_config, &desc, 1, I2CM_Transfer_cb, NULL);
	qurt_signal_wait(&i2c_ready_signal, I2CM_READY_SIG_MASK, QURT_SIGNAL_ATTR_CLEAR_MASK);

	return reg_val;
}

uint16_t humidity_temperature_read_sensor_reg16(qapi_I2CM_Config_t *pI2C_config, uint8 reg_addr_val)
{
	qapi_Status_t status;
	uint8 reg_addr;
	uint16_t  reg_val;
	qapi_I2CM_Descriptor_t desc[2];

	reg_addr = reg_addr_val | 0x80;
	desc[0].buffer = &reg_addr;
	desc[0].length = 1;
	desc[0].transferred = 0;
	desc[0].flags = QAPI_I2C_FLAG_START | QAPI_I2C_FLAG_WRITE;

	desc[1].buffer = (uint8_t *)&reg_val;
	desc[1].length = 2;
	desc[1].transferred = 0;
	desc[1].flags = QAPI_I2C_FLAG_START | QAPI_I2C_FLAG_STOP | QAPI_I2C_FLAG_READ;
	status = qapi_I2CM_Transfer(h1, pI2C_config, desc, 2, I2CM_Transfer_cb, NULL);
	if (status != QAPI_OK)
	{
		return 0;
	}

	qurt_signal_wait(&i2c_ready_signal, I2CM_READY_SIG_MASK, QURT_SIGNAL_ATTR_CLEAR_MASK);

	return reg_val;
}

int32_t Sensors_Humidity_Temperature_Get_Measured_Value()
{
	qapi_Status_t status;
	uint16_t   T0_DegCx8, T1_DegCx8, T_current;
	uint8_t    T0_T1_MSB;
	int16_t    T0_OUT_val, T1_OUT_val, T_OUT_val;
	uint32_t   T0_DegCx8_f, T1_DegCx8_f, T_DegCx8_fx10;
	uint16_t   H0_rHx2, H1_rHx2, H_current;
	uint32_t   H_rHx2_fx10;
	int16_t    H0_T0_OUT_val, H1_T0_OUT_val, H_OUT_val;

	status = qapi_I2CM_Open(QAPI_I2CM_INSTANCE_002_E, &h1);
	if (status != QAPI_OK)
	{
		QCLI_Printf(qcli_humidity_temperature_handle, "  ------  qapi_I2CM_Open fail ------\n");
		return QAPI_ERROR;
	}
	qurt_signal_init(&i2c_ready_signal);
	write_humidity_temperature_reg(HUMIDITY_TEMPERATURE_I2C_REG_ADDR_CTRL_1,0x83);
	// read temperature
	QCLI_Printf(qcli_humidity_temperature_handle, "  ------AITg Temperature ------\n");
	T_OUT_val = humidity_temperature_read_sensor_reg16(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_T_OUT);
	QCLI_Printf(qcli_humidity_temperature_handle, "T:%d\n",T_OUT_val);
	T0_DegCx8 = read_sensor_reg8(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_T0_DegCx8);
	T1_DegCx8 = read_sensor_reg8(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_T1_DegCx8);
	T0_T1_MSB = read_sensor_reg8(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_T0_T1_MSB);
	QCLI_Printf(qcli_humidity_temperature_handle, "DegCx8 T0:0x%x  T1:0x%x  T0_T_MSB:%x\n", T0_DegCx8, T1_DegCx8, T0_T1_MSB);

	T0_DegCx8 |= ((uint16_t)(T0_T1_MSB & 3)) << 8;
	T1_DegCx8 |= ((uint16_t)((T0_T1_MSB >> 2) & 3)) << 8;
	QCLI_Printf(qcli_humidity_temperature_handle, "DegCx8 with MSB T0:0x%x  T1:0x%x\n", T0_DegCx8, T1_DegCx8);

	T0_OUT_val = humidity_temperature_read_sensor_reg16(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_T0_OUT);
	T1_OUT_val = humidity_temperature_read_sensor_reg16(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_T1_OUT);
	QCLI_Printf(qcli_humidity_temperature_handle, "register 2's comp T0:%d T1:%d T:%d\n", T0_OUT_val, T1_OUT_val, T_OUT_val);

	T0_DegCx8_f = T0_DegCx8;
	T1_DegCx8_f = T1_DegCx8;
	T_DegCx8_fx10 = (T1_DegCx8_f - T0_DegCx8_f)  * (T_OUT_val - T0_OUT_val) * 10 / (T1_OUT_val - T0_OUT_val) + T0_DegCx8_f * 10;

	QCLI_Printf(qcli_humidity_temperature_handle, "DegCx8 T0:%d T1:%d Tx10:%d\n", T0_DegCx8_f, T1_DegCx8_f, T_DegCx8_fx10);
	T_current = T_DegCx8_fx10 / 8;
	QCLI_Printf(qcli_humidity_temperature_handle, "Current Temperature:%d.%d\n", T_current / 10, T_current % 10);


	// read humidity
	QCLI_Printf(qcli_humidity_temperature_handle, "\n------  Relative Humidity ------\n");
	H_OUT_val = humidity_temperature_read_sensor_reg16(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_H_OUT);

	H0_rHx2 = read_sensor_reg8(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_H0_rHx2);
	H1_rHx2 = read_sensor_reg8(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_H1_rHx2);
	QCLI_Printf(qcli_humidity_temperature_handle, "rHx2 H0:0x%x  H1:0x%x\n", H0_rHx2, H1_rHx2);

	H0_T0_OUT_val = humidity_temperature_read_sensor_reg16(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_H0_T0_OUT);
	H1_T0_OUT_val = humidity_temperature_read_sensor_reg16(&config_humidity_temperature, HUMIDITY_TEMPERATURE_I2C_REG_ADDR_H1_T0_OUT);
	QCLI_Printf(qcli_humidity_temperature_handle, "2's comp H0_T0:%d H1_T0:%d H:%d\n", H0_T0_OUT_val, H1_T0_OUT_val, H_OUT_val);

	H_rHx2_fx10 = (H1_rHx2 - H0_rHx2)  * (H_OUT_val - H0_T0_OUT_val) * 10 / (H1_T0_OUT_val - H0_T0_OUT_val) + H0_rHx2 * 10;

	QCLI_Printf(qcli_humidity_temperature_handle, "rHx2 Hx10:%d\n", H_rHx2_fx10);
	H_current = H_rHx2_fx10 / 2;
	QCLI_Printf(qcli_humidity_temperature_handle, "Current rH:%d.%d%% rH\n", H_current / 10, H_current % 10);
	QCLI_Printf(qcli_humidity_temperature_handle, "  ----------------------------------------------------\n");
	qurt_signal_destroy(&i2c_ready_signal);
	status = qapi_I2CM_Close(h1);
	if (status != QAPI_OK)
	{
		QCLI_Printf(qcli_humidity_temperature_handle, "  ------  qapi_I2CM_Close fail ------\n");
		return QAPI_ERROR;
	}
	return 0;
}

void Initialize_Humidity_Temperature_Demo(void)
{
	qapi_Status_t status;
	qcli_humidity_temperature_handle = QCLI_Register_Command_Group(NULL, &humidity_temperature_cmd_group);
	if (qcli_humidity_temperature_handle)
	{
		QCLI_Printf(qcli_humidity_temperature_handle, "qcli_humidity_temperature_handle Registered\n");
	}

	//I2C Scan Slave Address
	for (int address = 0x01; address <= 0x7f/*0x7f*/; address++)
	{
		status = qapi_I2CM_Open(QAPI_I2CM_INSTANCE_002_E, &h1);
		if (status != QAPI_OK)
		{
			QCLI_Printf(qcli_humidity_temperature_handle, "  ------  qapi_I2CM_Open fail ------\n");
		}
		qurt_signal_init(&i2c_ready_signal);
		do
		{
			qapi_I2CM_Descriptor_t desc;
			uint8 reg_val;
			qapi_I2CM_Config_t config_humidity_temperature_addr =
			{
				100,            /**< I2C bus speed in kHz. */
				address,        /**< 7-bit I2C slave address. */
				0,              /**< SMBUS mode transfers. Set to TRUE for SMBUS mode. */
				0,   			/**< Maximum slave clock stretch in us that a slave might perform. */
				0,         		/**< Core Specific Configuration. Recommended 0. */
				0,         		/**< Core Specific Configuration. Recommended 0. */
			};

			desc.buffer = &reg_val;
			desc.length = 1;
			desc.transferred = 0;
			desc.flags = QAPI_I2C_FLAG_START |QAPI_I2C_FLAG_READ | QAPI_I2C_FLAG_STOP;
			status = qapi_I2CM_Transfer (h1, &config_humidity_temperature_addr, &desc, 1, I2CM_Transfer_cb, NULL);
			if (status != QAPI_OK)
			{
				QCLI_Printf(qcli_humidity_temperature_handle, "  ------  qapi_I2CM_Transfer fail ------\n");
			}
			qurt_signal_wait(&i2c_ready_signal, I2CM_READY_SIG_MASK, QURT_SIGNAL_ATTR_CLEAR_MASK);
			if (desc.flags == 0x0a)
			{
				QCLI_Printf(qcli_humidity_temperature_handle, "AITg I2C Slave address 0x%x\n",address);
			}
		}while(0);

		qurt_signal_destroy(&i2c_ready_signal);
		status = qapi_I2CM_Close(h1);
		if (status != QAPI_OK)
		{
			QCLI_Printf(qcli_humidity_temperature_handle, "  ------  qapi_I2CM_Close fail ------\n");
		}
	}
}
